// * ************************************************************************ 
// *         Test for standard file dialogs                                 *
// * ************************************************************************

#include <amulet.h>

#include "file_dialog_widgets.h"

#include <am_inc.h>
#include OPAL_ADVANCED__H
#include STR_STREAM__H


Am_Slot_Key main_menu = Am_Register_Slot_Name ("main_menu");
Am_Slot_Key main_file = Am_Register_Slot_Name ("main_file");
Am_Slot_Key main_dir = Am_Register_Slot_Name ("main_dir");

Am_Object CreateMainWindow(char *fileName, char *dirName);

int stagger_x = 20;
int stagger_y = 50;

static void stagger_win(Am_Object& win) 
{
  if (!(win.Valid)) return;

  stagger_x += 20;
  stagger_y += 20;

  if (stagger_x > ((int) Am_Screen.Get(Am_WIDTH))/2)
    stagger_x = 20;
  if (stagger_y > ((int) Am_Screen.Get(Am_HEIGHT))/2)
    stagger_y = 20;

  win.Set(Am_LEFT, stagger_x);
  win.Set(Am_TOP, stagger_y);
}


//*********************************************************************
//
//   myQuit
//
//*********************************************************************
Am_Define_Method(Am_Object_Method, void, myQuit, (Am_Object /*self*/))
{
  Am_Exit_Main_Event_Loop();
}

//*********************************************************************
//
//   myNew
//
//*********************************************************************
Am_Define_Method(Am_Object_Method, void, myNew, (Am_Object /*self*/))
{
  Am_Screen.Add_Part(CreateMainWindow(NULL, NULL));
}


//*********************************************************************
//
//   myOpen
//
//*********************************************************************
Am_Define_Method(Am_Object_Method, void, myOpen, (Am_Object /*self*/))
{
#if defined(_WINDOWS)

  static char      szFileName[_MAX_PATH] ;
  static char      szTitleName[_MAX_FNAME + _MAX_EXT] ;

  Am_Object  win = self.Get_Owner().Get_Owner().Get_Owner();
  //Am_Drawonable *drawonable = (Am_Drawonable::Narrow((Am_Ptr) &win));
  //Am_POP_UP_ERROR_WINDOW("Drawonable " << (long) drawonable);


  HWND hwnd = GetActiveWindow();
  Am_Drawonable *drawonable = Am_Drawonable::Narrow(m_mapWnd2Drw.GetAt(hwnd));
  if (drawonable)
    drawonable->Set_Want_Move(true);
  
  Am_POP_UP_ERROR_WINDOW("Active window " << (long) hwnd);
  
  PopFileInitialize(hwnd);
  if (PopFileOpenDlg (hwnd, szFileName, szTitleName))  {
    /*
      if (!PopFileRead (hwndEdit, szFileName)) {
      OkMessage (hwnd, "Could not read file %s!",
      szTitleName) ;
      szFileName[0]  = '\0' ;
      szTitleName[0] = '\0' ;
      }
      */
  }
	
#elif defined(_MACINTOSH)
	 
#else
  Am_String fileOnly("");
  Am_String fullPath = Am_Get_Full_Filename(".");
  Am_Object theDialog = InitializeGetFileDialog(fullPath, fileOnly);
  Am_String theFile;

  Am_Value value = Am_Show_Dialog_And_Wait(theDialog, true);
  if ((value.Valid()) && (strcmp("OK", (char*) ((Am_String) value)) == 0)) {
    theFile = ((Am_Object) theDialog.Get(fileDialog_txtSelection)).Get(Am_VALUE);
    Am_POP_UP_ERROR_WINDOW("Open the file " << theFile);    
    Am_Screen.Add_Part(CreateMainWindow(NULL, (char*) theFile));
  } else    
    Am_POP_UP_ERROR_WINDOW("Canceled by user");

  theDialog.Destroy();
#endif
}

//*********************************************************************
//
//   mySave
//
//*********************************************************************
Am_Define_Method(Am_Object_Method, void, mySave, (Am_Object /*self*/))
{
   ;
}

//*********************************************************************
//
//   mySaveAs
//
//*********************************************************************
Am_Define_Method(Am_Object_Method, void, mySaveAs, (Am_Object /*self*/))
{
#if defined(_WINDOWS) 
	
	
#elif defined(_MACINTOSH)
	 
#else
  Am_String startDir = Am_Get_Full_Filename(".");
  Am_Object theDialog = InitializePutFileDialog(startDir, "");
  Am_String theFile;

  Am_Value value = Am_Show_Dialog_And_Wait(theDialog, true);
  if ((value.Valid()) && (strcmp("OK", (char*) ((Am_String) value)) == 0)) {
    theFile = ((Am_Object) theDialog.Get(fileDialog_txtSelection)).Get(Am_VALUE);  
    Am_POP_UP_ERROR_WINDOW("Save as file " << theFile);
  } else    
    Am_POP_UP_ERROR_WINDOW("Canceled by user");

  theDialog.Destroy();
#endif
}


//*********************************************************************
//
//   myMenuClick
//
//*********************************************************************
Am_Define_Method(Am_Object_Method, void, myMenuClick, (Am_Object /*self*/))
{
  ;
}

//*********************************************************************
//
//   CreateMenuBar
//
//*********************************************************************
Am_Object CreateMenuBar()
{
  Am_Object my_menu_bar = Am_Menu_Bar.Create()
    .Set (Am_FILL_STYLE, Am_Motif_Gray)
    .Set (Am_ITEMS, Am_Value_List ()
	  .Add (Am_Command.Create("File_Commands")
		.Set(Am_LABEL, "File")
		.Set(Am_DO_METHOD, myMenuClick)
		.Set(Am_ITEMS, Am_Value_List ()
		     .Add(Am_Command.Create("New_Command")
			  .Set(Am_DO_METHOD, myNew)
			  .Set(Am_LABEL, "New"))
		     .Add(Am_Command.Create("Open_Command")
			  .Set(Am_DO_METHOD, myOpen)
			  .Set(Am_LABEL, "Open..."))
		     .Add(Am_Command.Create("Save_Command")
			  .Set(Am_DO_METHOD, mySave)
			  .Set(Am_LABEL, "Save"))
		     .Add(Am_Command.Create("SaveAs_Command")
			  .Set(Am_DO_METHOD, mySaveAs)
			  .Set(Am_LABEL, "Save as..."))
		     
		     .Add(Am_Menu_Line_Command.Create())
		     .Add(Am_Command.Create("Quit_Command")
			  .Set(Am_DO_METHOD, myQuit)
			  .Set(Am_LABEL, "Quit")
			  .Set(Am_ACCELERATOR, "^Q"))
		     )
		)
	  .Add (Am_Command.Create("Edit_Command")
		.Set(Am_LABEL, "Edit")
		.Set(Am_DO_METHOD, myMenuClick)
		.Set(Am_ITEMS, Am_Value_List ()
		     .Add ("Undo")
		     .Add ("Cut")
		     .Add ("Copy")
		     .Add ("Paste")
		     .Add ("Clear"))
		)
	  );
  return my_menu_bar;
}

//*********************************************************************
//
//   CreateMainWindow
//
//*********************************************************************
Am_Object CreateMainWindow(char *fileName, char *dirName)
{
  Am_Object my_win = Am_Window.Create ("my_win")
    .Set (Am_LEFT, 20)
    .Set (Am_TOP, 50)
    .Set (Am_WIDTH, ((int) Am_Screen.Get(Am_WIDTH)) / 2)
    .Set (Am_HEIGHT,((int) Am_Screen.Get(Am_HEIGHT)) / 2);

  stagger_win(my_win);

  Am_Image_Array my_image("amulet.gif");

  my_win.Add_Part(main_menu, CreateMenuBar());
  my_win.Add(main_file, fileName);
  my_win.Add(main_dir, dirName);

  return my_win;
}

//*********************************************************************
//
//   main
//
//*********************************************************************
int main (void)
{
  Am_Initialize ();

  Am_String filename = "file_dialog_widgets.cpp";
  Am_String ext = Am_Get_File_Type(filename);
  cout << "ext: " << ext << endl << flush;

  bool exists = Am_File_Exists(filename);
  cout << "exists: " << exists << endl << flush;

  if (exists) {
    long size = Am_Get_File_Size(filename);
    cout << "size: " << size << endl << flush;
  }

  bool doesnt_exist = Am_File_Exists("better not exist");
  cout << "doesnt_exist: " << doesnt_exist << endl << flush;

  bool no = Am_File_Is_Directory(filename);
  cout << "no: " << no << endl << flush;

  Am_String pathname = Am_Get_Full_Filename(filename);
  cout << "pathname: " << pathname << endl << flush;

  Am_String name_only;
  Am_String directory = Am_Get_Directory_Part(pathname, name_only);
  cout << "name_only: " << name_only << endl << flush;
  cout << "directory: " << directory << endl << flush;

  Am_String name_together = Am_Concat_Filename(directory, name_only);
  cout << "name_together: " << name_together << endl << flush;

  Am_Value_List l = Am_Parse_Filename(directory);
  cout << "l: " << l << endl << flush;

  Am_Value_List m = Am_Get_Directory_Contents(directory);
  cout << "m: " << m << endl << flush;

  bool yes = Am_File_Is_Directory(directory);
  cout << "yes: " << yes << endl << flush;

  long dsize = Am_Get_File_Size(directory);
  cout << "dsize: " << dsize << endl << flush;

  Am_Screen.Add_Part(CreateMainWindow(NULL, NULL));

  /*****************************/

  Am_Main_Event_Loop ();

  Am_Cleanup ();

  return 0;
}
